package org.dromara.system.service.impl;

import org.dromara.common.core.utils.MapstructUtils;
import org.dromara.common.core.utils.StringUtils;
import org.dromara.common.mybatis.core.page.TableDataInfo;
import org.dromara.common.mybatis.core.page.PageQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.dromara.system.domain.bo.TblDeviceBo;
import org.dromara.system.domain.vo.TblDeviceVo;
import org.dromara.system.domain.TblDevice;
import org.dromara.system.mapper.TblDeviceMapper;
import org.dromara.system.service.ITblDeviceService;

import java.util.List;
import java.util.Map;
import java.util.Collection;

/**
 * 设备-摄像机Service业务层处理
 *
 * @author Lion Li
 * @date 2025-07-01
 */
@Slf4j
@RequiredArgsConstructor
@Service
public class TblDeviceServiceImpl implements ITblDeviceService {

    private final TblDeviceMapper baseMapper;

    /**
     * 查询设备-摄像机
     *
     * @param id 主键
     * @return 设备-摄像机
     */
    @Override
    public TblDeviceVo queryById(Long id){
        return baseMapper.selectVoById(id);
    }

    /**
     * 分页查询设备-摄像机列表
     *
     * @param bo        查询条件
     * @param pageQuery 分页参数
     * @return 设备-摄像机分页列表
     */
    @Override
    public TableDataInfo<TblDeviceVo> queryPageList(TblDeviceBo bo, PageQuery pageQuery) {
        LambdaQueryWrapper<TblDevice> lqw = buildQueryWrapper(bo);
        Page<TblDeviceVo> result = baseMapper.selectVoPage(pageQuery.build(), lqw);
        return TableDataInfo.build(result);
    }

    /**
     * 查询符合条件的设备-摄像机列表
     *
     * @param bo 查询条件
     * @return 设备-摄像机列表
     */
    @Override
    public List<TblDeviceVo> queryList(TblDeviceBo bo) {
        LambdaQueryWrapper<TblDevice> lqw = buildQueryWrapper(bo);
        return baseMapper.selectVoList(lqw);
    }

    private LambdaQueryWrapper<TblDevice> buildQueryWrapper(TblDeviceBo bo) {
        Map<String, Object> params = bo.getParams();
        LambdaQueryWrapper<TblDevice> lqw = Wrappers.lambdaQuery();
        lqw.orderByAsc(TblDevice::getId);
        lqw.like(StringUtils.isNotBlank(bo.getName()), TblDevice::getName, bo.getName());
        lqw.eq(StringUtils.isNotBlank(bo.getDirection()), TblDevice::getDirection, bo.getDirection());
        lqw.eq(StringUtils.isNotBlank(bo.getZh()), TblDevice::getZh, bo.getZh());
        lqw.eq(StringUtils.isNotBlank(bo.getLon()), TblDevice::getLon, bo.getLon());
        lqw.eq(StringUtils.isNotBlank(bo.getLat()), TblDevice::getLat, bo.getLat());
        lqw.eq(StringUtils.isNotBlank(bo.getExt1()), TblDevice::getExt1, bo.getExt1());
        lqw.eq(StringUtils.isNotBlank(bo.getExt2()), TblDevice::getExt2, bo.getExt2());
        if(params.get("hasevent")!=null && params.get("hasevent").equals("1")){
            lqw.apply("(select count(tbl_event.id) from tbl_event where tbl_device.id=tbl_event.ext2->'$.deviceVo.id' and tbl_event.status='2')>0");
        }
        return lqw;
    }

    /**
     * 新增设备-摄像机
     *
     * @param bo 设备-摄像机
     * @return 是否新增成功
     */
    @Override
    public Boolean insertByBo(TblDeviceBo bo) {
        TblDevice add = MapstructUtils.convert(bo, TblDevice.class);
        validEntityBeforeSave(add);
        boolean flag = baseMapper.insert(add) > 0;
        if (flag) {
            bo.setId(add.getId());
        }
        return flag;
    }

    /**
     * 修改设备-摄像机
     *
     * @param bo 设备-摄像机
     * @return 是否修改成功
     */
    @Override
    public Boolean updateByBo(TblDeviceBo bo) {
        TblDevice update = MapstructUtils.convert(bo, TblDevice.class);
        validEntityBeforeSave(update);
        return baseMapper.updateById(update) > 0;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(TblDevice entity){
        //TODO 做一些数据校验,如唯一约束
    }

    /**
     * 校验并批量删除设备-摄像机信息
     *
     * @param ids     待删除的主键集合
     * @param isValid 是否进行有效性校验
     * @return 是否删除成功
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if(isValid){
            //TODO 做一些业务上的校验,判断是否需要校验
        }
        return baseMapper.deleteByIds(ids) > 0;
    }
}
